/**
 * @file FileFolderShellInteraction.cs
 * @author Yarrago
 * @note Copyright  Yarrago (yarrago.com) 2010
 * @version 1.0.0
 * @date Created: 29/11/2010
 * @date Modified: 29/11/2010
 * @note Status: Complete
 * 
 * @note Modified by Name on --/--/2010. Description:
 * 
 * 
 * 
 * 
 * 
 * License: "Modified BSD License" 3-clause license.
 * 
 * Copyright (c) 2010, Yarrago (yarrago.com)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *   - Redistributions of source code must retain the above copyright notice, this list of 
 *     conditions and the following disclaimer.
 *   - Redistributions in binary form must reproduce the above copyright notice, this list of 
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   - Neither the name of the yarrago.com nor the names of its contributors may be used to endorse
 *     or promote products derived from this software without specific prior written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace ShellInteraction
{
    public class FileFolderShellInteraction
    {
        private enum WFunc : uint // UInt32?
        {
            FO_MOVE = 0x0001,
            FO_COPY = 0x0002,
            FO_DELETE = 0x0003,
            FO_RENAME = 0x0004
        }

        private enum FILEOP_FLAGS : ushort
        {
            FOF_MULTIDESTFILES = 0x0001,
            FOF_CONFIRMMOUSE = 0x0002,
            FOF_SILENT = 0x0004,
            FOF_RENAMEONCOLLISION = 0x0008,
            FOF_NOCONFIRMATION = 0x0010,
            FOF_WANTMAPPINGHANDLE = 0x0020,
            FOF_ALLOWUNDO = 0x0040,
            FOF_FILESONLY = 0x0080,
            FOF_SIMPLEPROGRESS = 0x0100,
            FOF_NOCONFIRMMKDIR = 0x0200,
            FOF_NOERRORUI = 0x0400,
            FOF_NOCOPYSECURITYATTRIBS = 0x0800,
            FOF_NORECURSION = 0x1000,
            FOF_NO_CONNECTED_ELEMENTS = 0x2000,
            FOF_WANTNUKEWARNING = 0x4000,
            FOF_NORECURSEREPARSE = 0x8000,
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        struct SHFILEOPSTRUCT
        {
            public IntPtr hwnd;
            public WFunc wFunc;                // UInt32?
            [MarshalAs(UnmanagedType.LPWStr)]
            public string pFrom;
            [MarshalAs(UnmanagedType.LPWStr)]
            public string pTo;
            public FILEOP_FLAGS fFlags;         // ushort?
            [MarshalAs(UnmanagedType.Bool)]
            public bool fAnyOperationsAborted;
            public IntPtr hNameMappings;
            [MarshalAs(UnmanagedType.LPWStr)]
            public string lpszProgressTitle;
        }
     
        [System.Runtime.InteropServices.DllImport("shell32.dll", CharSet = CharSet.Unicode)]
        static extern int SHFileOperation([In] ref SHFILEOPSTRUCT lpFileOp);

        /// <summary>
        /// Moves a file to the recycling bin.
        /// 
        /// Note: If I ever add an overload that takes a string warn the user that the path must be fully qualified (absolute),
        /// relative files are not relative to the same location as the rest of the application. 
        /// </summary>
        /// <param name="f">The file or folder to delete.</param>
        /// <param name="owner">The owner of the modal warning dialoge. (Can be null if we arn't warning the user).</param>
        /// <param name="warnUser">Whether to warn the user we are about to delete a file.</param>
        /// <returns></returns>
        static public bool recycle(System.IO.FileSystemInfo f, IntPtr owner, bool warnUser)
        {
            if (f != null)
            {
                SHFILEOPSTRUCT fileOp = new SHFILEOPSTRUCT();

                fileOp.wFunc = WFunc.FO_DELETE;
                fileOp.fFlags = FILEOP_FLAGS.FOF_ALLOWUNDO;

                if (warnUser && (owner != null))
                {
                    fileOp.hwnd = owner;
                }
                else
                {
                    fileOp.fFlags |= FILEOP_FLAGS.FOF_NOCONFIRMATION;
                }

                fileOp.pFrom += f.FullName + "\0";          // Double null terminate.

                return (SHFileOperation(ref fileOp) == 0);
            }
            else
            {
                return false;
            }
        }
    }
}
