/** 
 * @file License.cs
 * @author Yarrago
 * @note Copyright  Yarrago (yarrago.com) 2010
 * @version 1.0.0
 * @date Created: 10/12/2010
 * @date Modified: 10/12/2010
 * 
 * @note Modified by Name on --/--/2010. Description:
 * 
 * 
 * 
 * 
 * 
 * License: "Modified BSD License" 3-clause license.
 * 
 * Copyright (c) 2010, Yarrago (yarrago.com)
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, are 
 * permitted provided that the following conditions are met:
 *   - Redistributions of source code must retain the above copyright notice, this list of 
 *     conditions and the following disclaimer.
 *   - Redistributions in binary form must reproduce the above copyright notice, this list of 
 *     conditions and the following disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   - Neither the name of the yarrago.com nor the names of its contributors may be used to endorse
 *     or promote products derived from this software without specific prior written permission.
 *   
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR 
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace License
{
    public partial class License : Form
    {
        private Form mainApplication = null;

        // Would be nice if we could read the licence from file on compile but don't know if C# provides that functionality.
        private string licenseText = 
            "License: \"Modified BSD License\" 3-clause license." + "\r\n" +
            "" + "\r\n" +
            "Copyright (c) 2010, Yarrago (yarrago.com)" + "\r\n" +
            "All rights reserved." + "\r\n" +
            "" + "\r\n" +
            "Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:" + "\r\n" +
            "  - Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer." + "\r\n" +
            "  - Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution." + "\r\n" +
            "  - Neither the name of the yarrago.com nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission." + "\r\n" +
            "" + "\r\n" +
            "THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS \"AS IS\" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.";

        /// <summary>
        /// Form cosnstructor, takes the main application form as a paramater.
        /// </summary>
        /// <param name="startForm">The main application form to run if the license is accepted.</param>
        public License(Form startForm)
        {
            mainApplication = startForm;
            InitializeComponent();
        }

        /// <summary>
        /// Form initialization function.
        /// </summary>
        private void License_Load(object sender, EventArgs e)
        {
            if (ScanHelper.Properties.Settings.Default.LicenseAccepted == true)
            {
                licenseAccpted();
            }
            else
            {
                licenseTextBox.Text = licenseText;
                licenseTextBox.Select(0, 0);
            }
        }

        /// <summary>
        /// Called when the accept button is clicked.
        /// </summary>
        private void acceptButton_Click(object sender, EventArgs e)
        {
            licenseAccpted();
        }

        /// <summary>
        /// Called when the decline button is clicked.
        /// </summary>
        private void declineButton_Click(object sender, EventArgs e)
        {
            licenseDeclined();
        }

        /// <summary>
        /// Called when the copy license button is clicked.
        /// </summary>
        private void copyLicenseButton_Click(object sender, EventArgs e)
        {
            if ((licenseText != null) && (licenseText != ""))
            {
                Clipboard.SetText(licenseText);
            }
            else
            {
                copyLicenseButton.Enabled = false;
            }
        }

        /// <summary>
        /// The license has been accepted start the application.
        /// </summary>
        private void licenseAccpted()
        {
            ScanHelper.Properties.Settings.Default.LicenseAccepted = true;
            ScanHelper.Properties.Settings.Default.Save();

            // This is a nasty way of doing this, there has to be a better way.
            this.Hide();

            mainApplication.ShowDialog();

            this.Close();
        }

        /// <summary>
        /// The license has been declined, close down.
        /// </summary>
        private void licenseDeclined()
        {
            this.Close();
            Application.Exit();
        }
    }
}